@@ -50,6 +50,7 @@
 #define ACCEL_PATH_NEW_TAB              ACCEL_PATH_ROOT "FileNewTab"
 #define ACCEL_PATH_NEW_WINDOW           ACCEL_PATH_ROOT "FileNewWindow"
 #define ACCEL_PATH_NEW_PROFILE          ACCEL_PATH_ROOT "FileNewProfile"
+#define ACCEL_PATH_SAVE_CONTENTS        ACCEL_PATH_ROOT "FileSaveContents"
 #define ACCEL_PATH_CLOSE_TAB            ACCEL_PATH_ROOT "FileCloseTab"
 #define ACCEL_PATH_CLOSE_WINDOW         ACCEL_PATH_ROOT "FileCloseWindow"
 #define ACCEL_PATH_COPY                 ACCEL_PATH_ROOT "EditCopy"
@@ -86,6 +87,7 @@
 #define KEY_PREV_TAB            CONF_KEYS_PREFIX "/prev_tab"
 #define KEY_RESET_AND_CLEAR     CONF_KEYS_PREFIX "/reset_and_clear"
 #define KEY_RESET               CONF_KEYS_PREFIX "/reset"
+#define KEY_SAVE_CONTENTS       CONF_KEYS_PREFIX "/save_contents"
 #define KEY_SET_TERMINAL_TITLE  CONF_KEYS_PREFIX "/set_window_title"
 #define KEY_TOGGLE_MENUBAR      CONF_KEYS_PREFIX "/toggle_menubar"
 #define KEY_ZOOM_IN             CONF_KEYS_PREFIX "/zoom_in"
@@ -104,14 +106,14 @@ typedef struct
   GClosure *closure;
   /* have gotten a notification from gtk */
   gboolean needs_gconf_sync;
-  gboolean gconf_writable;
+  gboolean accel_path_unlocked;
 } KeyEntry;
 
 typedef struct
 {
   KeyEntry *key_entry;
   guint n_elements;
-  gchar *user_visible_name;
+  const char *user_visible_name;
 } KeyEntryList;
 
 static KeyEntry file_entries[] =
@@ -122,6 +124,8 @@ static KeyEntry file_entries[] =
     KEY_NEW_WINDOW, ACCEL_PATH_NEW_WINDOW, GDK_SHIFT_MASK | GDK_CONTROL_MASK, GDK_n, NULL, FALSE, TRUE },
   { N_("New Profile"),
     KEY_NEW_PROFILE, ACCEL_PATH_NEW_PROFILE, 0, 0, NULL, FALSE, TRUE },
+  { N_("Save Contents"),
+    KEY_SAVE_CONTENTS, ACCEL_PATH_SAVE_CONTENTS, 0, 0, NULL, FALSE },
   { N_("Close Tab"),
     KEY_CLOSE_TAB, ACCEL_PATH_CLOSE_TAB, GDK_SHIFT_MASK | GDK_CONTROL_MASK, GDK_w, NULL, FALSE, TRUE },
   { N_("Close Window"),
@@ -286,7 +290,7 @@ key_from_gconf_key (const char *gconf_key)
 {
   const char *last_slash = strrchr (gconf_key, '/');
   if (last_slash)
-    return last_slash++;
+    return ++last_slash;
   return NULL;
 }
 
@@ -335,6 +339,8 @@ terminal_accels_init (void)
           gconf_client_notify (conf, key_entry->gconf_key);
 	}
     }
+
+  g_object_unref (conf);
   
   g_signal_connect (notification_group, "accel-changed",
                     G_CALLBACK (accel_changed_callback), NULL);
@@ -395,7 +401,6 @@ keys_change_notify (GConfClient *client,
   KeyEntry *key_entry;
   GdkModifierType mask;
   guint keyval;
-  gboolean gconf_writable;
 
   _terminal_debug_print (TERMINAL_DEBUG_ACCELS,
                          "key %s changed\n",
@@ -438,15 +443,9 @@ keys_change_notify (GConfClient *client,
   key_entry->gconf_keyval = keyval;
   key_entry->gconf_mask = mask;
 
-  gconf_writable = gconf_entry_get_is_writable (entry);
-  if (gconf_writable != key_entry->gconf_writable)
-    {
-      if (gconf_writable)
-        gtk_accel_map_unlock_path (key_entry->accel_path);
-      else
-        gtk_accel_map_lock_path (key_entry->accel_path);
-    }
-  key_entry->gconf_writable = gconf_writable;
+  /* Unlock the path, so we can change its accel */
+  if (!key_entry->accel_path_unlocked)
+    gtk_accel_map_unlock_path (key_entry->accel_path);
 
   /* sync over to GTK */
   _terminal_debug_print (TERMINAL_DEBUG_ACCELS,
@@ -460,6 +459,11 @@ keys_change_notify (GConfClient *client,
                               TRUE);
   inside_gconf_notify -= 1;
 
+  /* Lock the path if the gconf key isn't writable */
+  key_entry->accel_path_unlocked = gconf_entry_get_is_writable (entry);
+  if (!key_entry->accel_path_unlocked)
+    gtk_accel_map_lock_path (key_entry->accel_path);
+
   /* This seems necessary to update the tree model, since sometimes the
    * notification on the notification_group seems not to be emitted correctly.
    * Without this change, when trying to set an accel to e.g. Alt-T (while the main
@@ -629,8 +633,8 @@ accel_set_func (GtkTreeViewColumn *tree_column,
   else
     g_object_set (cell,
                   "visible", TRUE,
-                  "sensitive", ke->gconf_writable,
-                  "editable", ke->gconf_writable,
+                  "sensitive", ke->accel_path_unlocked,
+                  "editable", ke->accel_path_unlocked,
                   "accel-key", ke->gconf_keyval,
                   "accel-mods", ke->gconf_mask,
 		  NULL);
