@@ -72,10 +72,14 @@ typedef enum
 #define TERMINAL_PROFILE_BACKGROUND_IMAGE_FILE  "background-image-file"
 #define TERMINAL_PROFILE_BACKGROUND_TYPE        "background-type"
 #define TERMINAL_PROFILE_BACKSPACE_BINDING      "backspace-binding"
+#define TERMINAL_PROFILE_BOLD_COLOR             "bold-color"
+#define TERMINAL_PROFILE_BOLD_COLOR_SAME_AS_FG  "bold-color-same-as-fg"
 #define TERMINAL_PROFILE_CURSOR_BLINK_MODE      "cursor-blink-mode"
 #define TERMINAL_PROFILE_CURSOR_SHAPE           "cursor-shape"
 #define TERMINAL_PROFILE_CUSTOM_COMMAND         "custom-command"
 #define TERMINAL_PROFILE_DEFAULT_SHOW_MENUBAR   "default-show-menubar"
+#define TERMINAL_PROFILE_DEFAULT_SIZE_COLUMNS   "default-size-columns"
+#define TERMINAL_PROFILE_DEFAULT_SIZE_ROWS      "default-size-rows"
 #define TERMINAL_PROFILE_DELETE_BINDING         "delete-binding"
 #define TERMINAL_PROFILE_EXIT_ACTION            "exit-action"
 #define TERMINAL_PROFILE_FONT                   "font"
@@ -85,6 +89,7 @@ typedef enum
 #define TERMINAL_PROFILE_PALETTE                "palette"
 #define TERMINAL_PROFILE_SCROLL_BACKGROUND      "scroll-background"
 #define TERMINAL_PROFILE_SCROLLBACK_LINES       "scrollback-lines"
+#define TERMINAL_PROFILE_SCROLLBACK_UNLIMITED   "scrollback-unlimited"
 #define TERMINAL_PROFILE_SCROLLBAR_POSITION     "scrollbar-position"
 #define TERMINAL_PROFILE_SCROLL_ON_KEYSTROKE    "scroll-on-keystroke"
 #define TERMINAL_PROFILE_SCROLL_ON_OUTPUT       "scroll-on-output"
@@ -97,7 +102,7 @@ typedef enum
 #define TERMINAL_PROFILE_USE_SYSTEM_FONT        "use-system-font"
 #define TERMINAL_PROFILE_USE_THEME_COLORS       "use-theme-colors"
 #define TERMINAL_PROFILE_VISIBLE_NAME           "visible-name"
-#define TERMINAL_PROFILE_WORD_CHARS             "word-chards"
+#define TERMINAL_PROFILE_WORD_CHARS             "word-chars"
 
 /* TerminalProfile object */
 
